{% load i18n %}

<a href="{{ url }}">
    <button class="btn btn-primary btn-sm" data-tooltip-toggle="skillfarm-tooltip" title="{% translate 'Show Skillfarm' %}">
        <i class="fa-solid fa-eye"></i>
    </button>
</a>
