{% load i18n %}
{% load sri %}

{% sri_static 'skillfarm/js/skillfarm.js' %}
{% sri_static 'skillfarm/js/modals/modal-system.js' %}
{% sri_static 'skillfarm/js/modals/skillfarm-confirm.js' %}
{% sri_static 'skillfarm/js/modals/skillfarm-skillset.js' %}

{% sri_static 'skillfarm/libs/slimselect/js/slimselect.min.js' %}
{% sri_static 'skillfarm/libs/slimselect/css/slimselect.css' %}

<script>
    let skillSelect = new SlimSelect({
        select: '#skillset',
        settings: {
            hideSelected: true,
            closeOnSelect: false,
            allowDeselect: true,
        }
    })
</script>
