"""Initialize the app"""

__version__ = "1.0.2"
__title__ = "Skillfarm"

__package_name__ = "aa-skillfarm"
__app_name__ = "skillfarm"
__esi_compatibility_date__ = "2025-11-06"
__app_name_useragent__ = "AA-Skillfarm"

__github_url__ = f"https://github.com/Geuthur/{__package_name__}"

__characters_operations__ = [
    "GetCharactersCharacterIdSkills",
    "GetCharactersCharacterIdSkillqueue",
]
