# Generated by Django 4.2.11 on 2024-10-22 21:21

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("skillfarm", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="skillfarmaudit",
            name="last_notification",
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="skillfarmaudit",
            name="notification_sent",
            field=models.BooleanField(default=False),
        ),
        migrations.DeleteModel(
            name="SkillFarmNotification",
        ),
    ]
