{% load i18n %}
{% load static %}
{% load evelinks %}
{% load humanize %}

<!-- Modal for viewing requests -->
<div class="modal fade" id="modal{{ name }}Container" tabindex="-1" role="dialog" aria-labelledby="modalDialogLabel">
    <div class="modal-dialog modal-xl" role="document">
        <div class="modal-content">
            <button type="button" class="btn-close position-absolute top-0 end-0 m-3" data-bs-dismiss="modal" aria-label="Close"></button>
            <div class="modal-header">
                <h4 class="modal-title" id="modal-title">-</h4>
            </div>
            <!-- Hidden input to store the URL -->
            <input type="hidden" id="modal-hidden-url" value="">

            <div class="modal-body">
                <div id="modal{{ name }}Loader">
                    <img src="{% static 'skillfarm/images/loading.gif' %}">
                </div>
                <div class="card border-0" id="modal{{ name }}Content" style="display: none !important;"></div>
            </div>

            <div class="modal-footer d-flex justify-content-between">
                <button type="button" class="btn btn-secondary" data-bs-dismiss="modal">{% trans "Close" %}</button>
            </div>
        </div>
    </div>
</div>
