{% extends 'skillfarm/base.html' %}
{% load i18n %}
{% load humanize %}
{% load static %}

{% block page_topic %}<h1 class="page-header text-center">{{ page_title }}</h1>{% endblock page_topic %}

{% block skillfarm_block %}
    <!-- Skillfarm Details -->
    <div class="card">
        <div class="card-header bg-primary rounded-top">
            <h3>{% translate "Skillfarm" %} </h3>
        </div>
        <div class="card-body bg-secondary tab-content rounded-bottom">
            <i class="fa-solid fa-exclamation-triangle"></i>
            <span class="text-muted font-weight-bold">{% trans "Data will only update on change trought the client" %}</span>
            <div class="tab-content mt-2">
                {% include 'skillfarm/partials/table/skillfarm.html' %}
            </div>
        </div>
    </div>
    <!-- Inactive Characters -->
    <div class="pt-4">
        <div class="card">
            <div class="card-header bg-primary rounded-top">
                <h3>{% translate "Inactive Characters" %} </h3>
            </div>
            <div class="card-body bg-secondary tab-content rounded-bottom">
                <div class="tab-content">
                    {% include 'skillfarm/partials/table/inactive.html' %}
                </div>
            </div>
        </div>
    </div>
    <!-- Modal for viewing requests -->
    {% include 'skillfarm/partials/modals/skillfarm-confirm.html' %}
    {% include 'skillfarm/partials/modals/skillfarm-skillset.html' %}
    {% include 'skillfarm/partials/modals/modal.html' with name="ViewSkill" %}

{% endblock %}

{% block extra_javascript %}
{% include 'skillfarm/bundles/modal-bundle.html' %}
<script>
    setupModal('#modalViewSkillContainer', 'ajax_skillview', '#modalViewSkillContent', '#modalViewSkillLoader');
</script>
{% include 'bundles/datatables-js-bs5.html' %}
{% include 'skillfarm/bundles/settings-bundle.html' %}
{% include 'skillfarm/bundles/skillfarm-bundle.html' %}
{% endblock %}
