{% extends 'skillfarm/base.html' %}
{% load i18n %}
{% load humanize %}
{% load static %}

{% block skillfarm_block %}
    <div class="card">
        <div class="card-header bg-primary">
            <h3 class="text-white">{% translate "Character Overview" %}</h3>
        </div>
        <div class="card-body bg-secondary">
            <table class="table table-dark table-striped table-hover w-100" id="skillfarm-overview">
                <br>
                <thead>
                    <th class="col-portrait"></th>
                    <th class="col-user">{% translate "Character" %}</th>
                    <th class="col-group">{% translate "Corporation" %}</th>
                    <th class="col-actions">{% translate "Actions" %}</th>
                </thead>
                <tbody>
                    <!-- Table Row -->
                </tbody>
            </table>
        </div>
    </div>
{% endblock %}

{% block extra_javascript %}
    {% include 'bundles/datatables-js-bs5.html' %}
    {% include 'skillfarm/bundles/settings-bundle.html' %}
    {% include 'skillfarm/bundles/overview-bundle.html' %}
{% endblock %}
