# Generated by Django 5.1.7 on 2025-03-18 13:21

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveuniverse", "0010_alter_eveindustryactivityduration_eve_type_and_more"),
        ("skillfarm", "0003_characterskill_name_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="EveTypePrice",
            fields=[
                ("name", models.CharField(blank=True, max_length=255, null=True)),
                (
                    "eve_type",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        primary_key=True,
                        serialize=False,
                        to="eveuniverse.evetype",
                    ),
                ),
                ("buy", models.DecimalField(decimal_places=2, max_digits=20)),
                ("sell", models.DecimalField(decimal_places=2, max_digits=20)),
                ("updated_at", models.DateTimeField()),
            ],
        ),
    ]
