{% extends 'allianceauth/base-bs5.html' %}
{% load i18n %}

{% block page_title %}
    {{ page_title}}
{% endblock %}

{% block header_nav_brand %}
    <a class="nav-link text-white link-light link-underline link-underline-opacity-0" href="{% url 'skillfarm:index' %}">Skillfarm</a>
{% endblock header_nav_brand %}

{% block header_nav_collapse_left %}
    {% include 'skillfarm/partials/navigation.html' %}
{% endblock header_nav_collapse_left %}

{% block header_nav_collapse_right %}
    {% include 'skillfarm/partials/administration.html' %}
{% endblock header_nav_collapse_right %}

{% block content %}
    {% include 'bundles/datatables-css-bs5.html' %}
    <div class="allianceauth-skillfarm">
        <div class="skillfarm-body">
        {% block skillfarm_block %}
            <!-- Content goes here -->
        {% endblock %}
        </div>
    </div>
{% endblock %}

{% block extra_javascript %}
{% endblock %}
