{% load i18n %}
{% load sri %}

{% sri_static 'skillfarm/css/skillfarm.css' %}

<script type="application/javascript">
	let SkillfarmSettings = {
		csrfToken: '{% csrf_token %}',
        characterPk: '{{ character_id }}',
        updateInterval: '{{ update_interval }}',
        // URLs
        DetailsUrl: '{% url "skillfarm:api:get_details" character_id=12345 %}',
        SkillSetupUrl: '{% url "skillfarm:api:get_skillsetup" character_id=12345 %}',
        OverviewUrl: '{% url "skillfarm:api:get_character_overview" %}',

        // Translations
        translations: {
            character: '{% translate "Character" %}',
            progress: '{% translate "Progress" %}',
            skillInfo: '{% translate "Skill Info" %}',
            lastUpdated: '{% translate "Last Updated" %}',
            filter: '{% translate "Filter" %}',
            actions: '{% translate "Actions" %}',
            errorLoadingData: '{% translate "Error loading data:" %}',
        },
	};
</script>
