{% load i18n %}
{% load navactive %}

<li class="nav-item">
    <a class="nav-link" href="{% url 'skillfarm:index' %}">{% translate 'Skillfarm' %}</a>
</li>

<li class="nav-item">
    <a class="nav-link" href="{% url 'skillfarm:calc' %}">{% translate 'Skillfarm Calculator' %}</a>
</li>
