{% load i18n %}
{% load humanize %}
{% load static %}
<div class="table-responsive">
    <table class="table table-dark table-striped table-hover w-100" id="skillfarm-inactive">
        <thead>
            <th class="col-character w-auto">{% translate "Character" %}</th>
            <th class="col-quue w-auto no-sort">{% translate "Progress" %}</th>
            <th class="col-skill w-auto no-sort">{% translate "Skill Info" %}</th>
            <th class="col-last-updated w-auto">{% translate "Last Updated" %}</th>
            <th class="col-filter w-auto no-sort">{% translate "Filter" %}</th>
            <th class="col-actions w-auto no-sort text-end">{% translate "Actions" %}</th>
        </thead>

        <tbody>
        </tbody>
    </table>
</div>
