<button type="button"
    class="btn btn-{{ settings.color }} btn-sm btn-square me-2"
    data-bs-toggle="modal"
    data-tooltip-toggle="skillfarm-tooltip"
    data-character-id="{{ queryset.character.character_id }}"
    data-title="{{ settings.title }}"
    data-text="{{ settings.text }}"
    data-bs-target="#{{ settings.modal }}"
    data-{{ settings.ajax }}="{{ settings.action }}"

    title="{{ settings.title }}"
    >
    <span class="{{ settings.icon }}"></span>
</button>
