# Generated by Django 4.2.11 on 2025-02-19 23:15

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("skillfarm", "0002_skillfarmaudit_last_notification_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="characterskill",
            name="name",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name="characterskillqueueentry",
            name="has_no_skillqueue",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="characterskillqueueentry",
            name="last_check",
            field=models.DateTimeField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="characterskillqueueentry",
            name="name",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name="skillfarmaudit",
            name="name",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name="skillfarmsetup",
            name="name",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name="skillfarmaudit",
            name="id",
            field=models.AutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]
