{% load i18n %}
{% load navactive %}

<li class="nav-item">
    <a class="nav-link py-0" href="{% url 'skillfarm:character_overview' %}">
        <span class="btn btn-success">{% translate 'Overview' %}</span>
    </a>
</li>

<li class="nav-item">
    <a class="nav-link py-0" href="{% url 'skillfarm:add_char' %}">
        <span class="btn btn-success">{% translate 'Add Character' %}</span>
    </a>
</li>
