@media all {
    .nav-pills .nav-link {
        transition: opacity 0.3s ease; /* Übergangseffekt */
    }

    .nav-pills .nav-link:not(.active) {
        opacity: 0.6; /* Starttransparenz für nicht aktive Links */
    }

    .nav-pills .nav-link:not(.active):hover {
        opacity: 1.0; /* Transparenz auf 100% beim Hover */
    }

    .progress-outer {
        position: relative;
        background: rgb(255 255 255); /* Changed from rgba */
        border-radius: 50px;
        box-shadow: 0 0 10px rgb(0 219 231 / 70%);
        overflow: hidden; /* Ensure the inner progress bar doesn't overflow */
    }

    .progress {
        height: 27px;
        margin: 0;
        overflow: visible;
        border-radius: 50px;
        background: rgb(234 237 243); /* Changed from rgba */
        box-shadow: inset 0 10px 10px rgb(244 245 250 / 90%);
    }

    .progress .progress-bar {
        border-radius: 50px;
    }

    .progress .progress-value {
        position: absolute;
        left: 50%;
        top: 50%;
        transform: translate(-50%, -50%);
        font-size: 16px;
        font-weight: bold;
        color: rgb(255 255 255); /* Modern notation */
        letter-spacing: 2px;
        -webkit-text-stroke: 1px rgb(0 0 0); /* Modern notation */
    }

    .progress-bar.active {
        animation: reverse progress-bar-stripes 0.40s linear infinite, animate-positive 2s;
    }

    @keyframes animate-positive {
        0% { width: 0%; }
    }

    table.table-hover > tbody > tr:nth-of-type(2n+1) > * {
        background-color: rgb(58 58 58 / 30%); /* Modern notation */
    }

    table.table-hover > tbody > tr:nth-of-type(2n+1):hover > * {
        box-shadow: inset 0 0 0 9999px rgb(0 0 0 / 20%); /* Modern notation */
    }

    table.table-hover > tbody > tr:nth-of-type(1n+1):hover > * {
        box-shadow: inset 0 0 0 9999px rgb(0 0 0 / 20%); /* Modern notation */
    }

    .table-stripe-green {
        background-color: rgb(0 255 42 / 10%) !important; /* Modern notation */
    }

    .table-stripe-info {
        background-color: rgb(0 162 255 / 10%) !important; /* Modern notation */
    }

    .table-stripe-warning {
        background-color: rgb(255 238 0 / 10%) !important; /* Modern notation */
    }

    .table-stripe-red {
        background-color: rgb(255 0 0 / 10%) !important; /* Modern notation */
    }

    /* Fix X-Editable Cancel Button */

    .editable-cancel {
        background-color: rgb(231 76 60); /* Modern notation */
    }

    .editable-cancel:hover {
        background-color: rgb(231 76 60 / 70%); /* Modern notation */
    }

    /* Calculator */
    .price-list {
        list-style-type: none;
        padding: 0;
    }

    .price-list li {
        display: flex;
        justify-content: space-between;
    }

    .item-name {
        flex: 1;
    }

    .price {
        flex: 1;
        text-align: right;
        font-weight: bold;
    }
}
